import numpy as np
import matplotlib.pyplot as plt
from multiprocessing import Pool, freeze_support

from utilities import *

N = 7
K = 9
rw_mean_list = np.array([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
m_list = np.array([2, 1, 3, 1, 2, 3, 3, 4, 2])


def plot_with_error(i):

    MARKERS = ["o", "D", "s", "^", "v", "p", "*"]
    COLORS = ["red", "red", "blue", "blue"]
    LINES = ["solid", "dashed", "solid", "dashed"]

    Reward = np.array(eval(("Reward"+str(i+1))))

    Regret = np.array(
        [(t+1) * mab.oc_rw - sum(np.mean(Reward, axis=0)[:t+1]) for t in range(T)])
    Regret_std = np.array(
        [np.std((t+1) * mab.oc_rw - np.sum(Reward[:, :t+1], axis=1)) for t in range(T)])

    ax.plot(np.arange(1, T+1), Regret, linewidth=3, color=COLORS[i], marker=MARKERS[i], markersize=8.0, markevery=every,
            linestyle=LINES[i], label=Name[i])

    ax.fill_between(np.arange(1, T+1), Regret - Regret_std, Regret + Regret_std,
                    facecolor=COLORS[i], edgecolor=COLORS[i], alpha=0.15)


T = 20000
episode = 200


every = int(T/10)

mab = MPMABSA(N, K, rw_mean_list, m_list, 'Bernoulli')

gamma = 1
xi = [2, 20]


if __name__ == "__main__":
    freeze_support()

    Name = ['UCI '+r'$\delta=2/T$', "UCI "+r'$\delta=20/T$',
            "HFD "+r'$\delta=2/T$',  'HFD '+r'$\delta=20/T$']

    with Pool(10) as pool:

        Reward1 = pool.starmap(
            OrchExplore_with_para, [[mab, xi[0], T, 'uci'] for i in range(episode)])
        Reward2 = pool.starmap(
            OrchExplore_with_para, [[mab, xi[1], T, 'uci'] for i in range(episode)])
        Reward3 = pool.starmap(
            OrchExplore_with_para, [[mab, xi[0], T, 'hfd'] for i in range(episode)])
        Reward4 = pool.starmap(
            OrchExplore_with_para, [[mab, xi[1], T, 'hfd'] for i in range(episode)])

    fig, ax = plt.subplots()

    for i in range(4):
        plot_with_error(i)

    # Set tick font size
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(18)

    plt.xlabel("Time Slots", fontsize=20)
    plt.ylabel("Regret", fontsize=20)
    plt.ticklabel_format(style='sci', axis='both',
                         scilimits=(0, 0), useMathText=True)
    plt.locator_params(axis='x', nbins=5)
    ax.legend(prop={"size": 20})
    plt.tight_layout()

    plt.savefig(
        'figures/'+f"hfd_vs_uci_Orch_Gaussian_gamma={gamma}_xi={xi}_T={T}_episode={episode}.pdf", format='pdf')

    plt.show()
