import numpy as np
import matplotlib.pyplot as plt
from multiprocessing import Pool, freeze_support

from utilities import *


def initfn():  # beware to init random seed for each thread
    np.random.seed()


N = 18
K = 20
rw_mean_list = np.array([1.2, 1.1, 4.2, 4.9, 4.5, 3.4, 5.0, 4.2,
                         5.1, 3.9, 4.8, 5.7, 3.7, 4.7, 3.2, 5.1, 4.4, 5.1, 4.9, 4.1])
rw_mean_list = 1/rw_mean_list
m_list = np.array([8, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                   1, 1, 1, 1, 1, 1, 1, 1], dtype=int)


T = 50000
episode = 200

every = int(T/10)

mab = MPMABSA(N, K, rw_mean_list, m_list, 'Bernoulli')

gamma = 0.1
xi = 1

# ETC_CUCB(mab, gamma, xi, T, 'uci')


def plot_with_error(i):

    MARKERS = ["o", "D", "s", "^", "v", "p", "*"]
    COLORS = ["#bc2d14", "#0e5ad3", "#22aa16",
              "#a011a3", "#d1ba0e", "#14ccc2", "#d67413"]
    LINES = ["solid", "dashed", "dashdot", "dotted",
             "solid", "dashed", "dashdot", "dotted"]

    Reward = np.array(eval(("Reward"+str(i+1))))

    Regret = np.array(
        [(t+1) * mab.oc_rw - sum(np.mean(Reward, axis=0)[:t+1]) for t in range(T)])
    Regret_std = np.array(
        [np.std((t+1) * mab.oc_rw - np.sum(Reward[:, :t+1], axis=1)) for t in range(T)])

    ax.plot(np.arange(1, T+1), Regret, linewidth=3, color=COLORS[i], marker=MARKERS[i], markersize=8.0, markevery=every,
            linestyle=LINES[i], label=Name[i])

    ax.fill_between(np.arange(1, T+1), Regret - Regret_std, Regret + Regret_std,
                    facecolor=COLORS[i], edgecolor=COLORS[i], alpha=0.15)


def plot_with_error_reward(i):

    MARKERS = ["o", "D", "s", "^", "v", "p", "*"]
    COLORS = ["#bc2d14", "#0e5ad3", "#22aa16",
              "#a011a3", "#d1ba0e", "#14ccc2", "#d67413"]
    LINES = ["solid", "dashed", "dashdot", "dotted",
             "solid", "dashed", "dashdot", "dotted"]

    Reward = np.array(eval(("Reward"+str(i+1))))

    Regret = np.array(
        [sum(np.mean(Reward, axis=0)[:t+1]) for t in range(T)])
    Regret_std = np.array(
        [np.std(np.sum(Reward[:, :t+1], axis=1)) for t in range(T)])

    ax.plot(np.arange(1, T+1), Regret, linewidth=3, color=COLORS[i], marker=MARKERS[i], markersize=8.0, markevery=every,
            linestyle=LINES[i], label=Name[i])

    ax.fill_between(np.arange(1, T+1), Regret - Regret_std, Regret + Regret_std,
                    facecolor=COLORS[i], edgecolor=COLORS[i], alpha=0.15)


if __name__ == "__main__":
    freeze_support()

    Name = ["OrchExplore", "MP-SE-SA", "ETC-UCB", "UCB", "TS", "SE"]

    with Pool(10, initializer=initfn) as pool:

        Reward1 = pool.starmap(
            OrchExplore, [[mab, T] for i in range(episode)])
        Reward2 = pool.starmap(
            MPSESA, [[mab, gamma, xi, T, 'uci'] for i in range(episode)])
        Reward3 = pool.starmap(
            ETC_CUCB, [[mab, gamma, xi, T, 'uci'] for i in range(episode)])

        # Reward4 = pool.starmap(
        #     UCB_com, [[mab, gamma, T] for i in range(episode)])
        # Reward5 = pool.starmap(
        #     TS_com, [[mab, T] for i in range(episode)])
        # Reward6 = pool.starmap(
        #     SE_com, [[mab, gamma, T] for i in range(episode)])

    fig, ax = plt.subplots()

    for i in range(3):
        plot_with_error(i)

    # Set tick font size
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(18)

    plt.xlabel("Time Slots", fontsize=20)
    plt.ylabel("Regret", fontsize=20)
    plt.ticklabel_format(style='sci', axis='both',
                         scilimits=(0, 0), useMathText=True)
    ax.legend(prop={"size": 20})
    plt.tight_layout()

    plt.savefig(
        'figures/'+f"4g5g_regret_gamma={gamma}_xi={xi}_T={T}_episode={episode}.pdf", format='pdf')

    plt.show()

    fig, ax = plt.subplots()

    for i in range(3):
        plot_with_error_reward(i)

    # Set tick font size
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        label.set_fontsize(18)

    plt.xlabel("Time Slots", fontsize=20)
    plt.ylabel("Total throughput", fontsize=20)
    plt.ticklabel_format(style='sci', axis='both',
                         scilimits=(0, 0), useMathText=True)
    ax.legend(prop={"size": 20})
    plt.tight_layout()

    plt.savefig(
        'figures/'+f"4g5g_reward_gamma={gamma}_xi={xi}_T={T}_episode={episode}.pdf", format='pdf')

    plt.show()
